import { parser } from 'lezer-css';
import { LezerLanguage, indentNodeProp, continuedIndent, foldNodeProp } from '@codemirror/next/language';
import { styleTags, tags } from '@codemirror/next/highlight';

/// A language provider based on the [Lezer CSS
/// parser](https://github.com/lezer-parser/css), extended with
/// highlighting and indentation information.
const cssLanguage = LezerLanguage.define({
    parser: parser.configure({
        props: [
            indentNodeProp.add({
                Declaration: continuedIndent()
            }),
            foldNodeProp.add({
                Block(subtree) { return { from: subtree.from + 1, to: subtree.to - 1 }; }
            }),
            styleTags({
                "import charset namespace keyframes": tags.definitionKeyword,
                "media supports": tags.controlKeyword,
                "from to": tags.keyword,
                NamespaceName: tags.namespace,
                KeyframeName: tags.labelName,
                TagName: tags.typeName,
                ClassName: tags.className,
                PseudoClassName: tags.constant(tags.className),
                not: tags.operatorKeyword,
                IdName: tags.labelName,
                "FeatureName PropertyName AttributeName": tags.propertyName,
                NumberLiteral: tags.number,
                KeywordQuery: tags.keyword,
                UnaryQueryOp: tags.operatorKeyword,
                callee: tags.keyword,
                "CallTag ValueName": tags.atom,
                Callee: tags.variableName,
                Unit: tags.unit,
                "UniversalSelector NestingSelector": tags.definitionOperator,
                AtKeyword: tags.keyword,
                MatchOp: tags.compareOperator,
                "ChildOp SiblingOp, LogicOp": tags.logicOperator,
                BinOp: tags.arithmeticOperator,
                Important: tags.modifier,
                Comment: tags.blockComment,
                ParenthesizedContent: tags.special(tags.name),
                ColorLiteral: tags.color,
                StringLiteral: tags.string,
                ":": tags.punctuation,
                "PseudoOp #": tags.derefOperator,
                "; ,": tags.separator,
                "( )": tags.paren,
                "[ ]": tags.squareBracket,
                "{ }": tags.brace
            })
        ]
    }),
    languageData: {
        commentTokens: { block: { open: "/*", close: "*/" } },
        indentOnInput: /^\s*\}$/
    }
});
/// Returns an extension that installs the CSS language.
function css() {
    return cssLanguage;
}

export { css, cssLanguage };
