import { Language, defineLanguageFacet, foldNodeProp, languageDataProp } from '@codemirror/next/language';
import { styleTags, tags } from '@codemirror/next/highlight';
import { parser } from 'lezer-markdown';
import { htmlLanguage } from '@codemirror/next/lang-html';

// FIXME add indentation (and possibly continue-list) support
/// Language support for Markdown/CommonMark.
const markdownLanguage = new class extends Language {
    constructor() {
        let data = defineLanguageFacet({ block: { open: "<!--", close: "-->" } });
        super(data, parser.configure({
            props: [
                styleTags({
                    "Blockquote/...": tags.quote,
                    HorizontalRule: tags.atom,
                    "ATXHeading/... SetextHeading/...": tags.heading,
                    "Comment CommentBlock": tags.comment,
                    Escape: tags.escape,
                    Entity: tags.character,
                    "Emphasis/...": tags.emphasis,
                    "StrongEmphasis/...": tags.strong,
                    "Link/... Image/...": tags.link,
                    InlineCode: tags.monospace,
                    URL: tags.url,
                    "HeaderMark HardBreak QuoteMark ListMark LinkMark EmphasisMark CodeMark": tags.processingInstruction,
                    "CodeInfo LinkLabel": tags.labelName,
                    LinkTitle: tags.string
                }),
                foldNodeProp.add(type => {
                    if (!type.is("Block") || type.is("Document"))
                        return undefined;
                    return (tree, state) => ({ from: state.doc.lineAt(tree.from).to, to: tree.to });
                }),
                languageDataProp.add({
                    Document: data
                })
            ],
            htmlParser: htmlLanguage.parser.configure({ dialect: "noMatch" }),
        }));
    }
};
/// Returns an extension that installs the Markdown language.
function markdown() {
    return markdownLanguage;
}

export { markdown, markdownLanguage };
