import { Tree, TreeFragment, NodeProp, Input, PartialParse, ParseContext } from "lezer-tree";
import { Parser, ParserConfig } from "lezer";
import { EditorState, Extension, Facet } from "@codemirror/next/state";
export declare const language: Facet<Language, readonly Language[]>;
export declare const languageDataProp: NodeProp<Facet<{
    [name: string]: any;
}, readonly {
    [name: string]: any;
}[]>>;
export declare function defineLanguageFacet(baseData?: {
    [name: string]: any;
}): Facet<{
    [name: string]: any;
}, readonly {
    [name: string]: any;
}[]>;
export declare class Language {
    readonly data: Facet<{
        [name: string]: any;
    }>;
    private field;
    readonly extension: Extension;
    parser: {
        startParse: (input: Input, startPos: number, context: ParseContext) => PartialParse;
    };
    protected constructor(data: Facet<{
        [name: string]: any;
    }>, parser: {
        startParse(input: Input, pos: number, context: EditorParseContext): PartialParse;
    }, extraExtensions?: Extension[]);
    getTree(state: EditorState): Tree;
    ensureTree(state: EditorState, upto: number, timeout?: number): Tree | null;
}
export declare class LezerLanguage extends Language {
    readonly parser: Parser;
    private constructor();
    static define(spec: {
        parser: Parser;
        languageData?: {
            [name: string]: any;
        };
    }): LezerLanguage;
    configure(options: ParserConfig): LezerLanguage;
    languageDataFacetAt(state: EditorState, pos: number): Facet<{
        [name: string]: any;
    }, readonly {
        [name: string]: any;
    }[]>;
}
export declare function syntaxTree(state: EditorState): Tree;
export declare class EditorParseContext implements ParseContext {
    private parser;
    readonly state: EditorState;
    fragments: readonly TreeFragment[];
    viewport: {
        from: number;
        to: number;
    };
    private parse;
    skipUntilInView(from: number, to: number): void;
    skipUntil(from: number, to: number, until: Promise<any>): void;
}
//# sourceMappingURL=language.d.ts.map