import { EditorView } from '@codemirror/next/view';
import { highlightStyle, tags } from '@codemirror/next/highlight';

const chalky = "#e5c07b", coral = "#e06c75", dark = "#5c6370", fountainBlue = "#56b6c2", green = "#98c379", invalid = "#ffffff", lightDark = "#7f848e", lightWhite = "#abb2bf", malibu = "#61afef", purple = "#c678dd", whiskey = "#d19a66", background = "#282c34", selection = "#405948", cursor = "#528bff";
const oneDarkTheme = EditorView.theme({
    $: {
        color: lightWhite,
        backgroundColor: background,
        "& ::selection": { backgroundColor: selection },
        caretColor: cursor
    },
    "$$focused $cursor": { borderLeftColor: cursor },
    "$$focused $selectionBackground": { backgroundColor: selection },
    $panels: { backgroundColor: background, color: lightWhite },
    "$panels.top": { borderBottom: "2px solid black" },
    "$panels.bottom": { borderTop: "2px solid black" },
    $searchMatch: {
        backgroundColor: "#72a1ff59",
        outline: "1px solid #457dff"
    },
    "$searchMatch.selected": {
        backgroundColor: "#6199ff2f"
    },
    $activeLine: { backgroundColor: "#2c313c" },
    $selectionMatch: { backgroundColor: "#aafe661a" },
    "$matchingBracket, $nonmatchingBracket": {
        backgroundColor: "#bad0f847",
        outline: "1px solid #515a6b"
    },
    $gutters: {
        backgroundColor: background,
        color: "#545868",
        border: "none"
    },
    "$gutterElement.lineNumber": { color: "inherit" },
    $foldPlaceholder: {
        backgroundColor: "none",
        border: "none",
        color: "#ddd"
    },
    $tooltip: {
        border: "1px solid #181a1f",
        backgroundColor: "#606862"
    },
    "$tooltip.autocomplete": {
        "& > ul > li[aria-selected]": { backgroundColor: background }
    }
}, { dark: true });
const oneDarkHighlighter = highlightStyle({ tag: tags.comment,
    color: lightDark }, { tag: tags.keyword,
    color: purple }, { tag: [tags.name, tags.deleted],
    color: coral }, { tag: [tags.operator, tags.operatorKeyword, tags.regexp],
    color: fountainBlue }, { tag: [tags.string, tags.inserted],
    color: green }, { tag: tags.propertyName,
    color: malibu }, { tag: [tags.color, tags.constant(tags.name), tags.standard(tags.name)],
    color: whiskey }, { tag: tags.definition(tags.name),
    color: lightWhite }, { tag: [tags.typeName, tags.className, tags.number, tags.changed],
    color: chalky }, { tag: tags.meta,
    color: dark }, { tag: tags.strong,
    fontWeight: "bold" }, // FIXME export a template for this from highlight
{ tag: tags.emphasis,
    fontStyle: "italic" }, { tag: tags.link,
    color: dark,
    textDecoration: "underline" }, { tag: tags.heading,
    fontWeight: "bold",
    color: coral }, { tag: [tags.atom, tags.bool],
    color: whiskey }, { tag: tags.invalid,
    color: invalid });
/// Extension to enable the One Dark theme.
const oneDark = [oneDarkTheme, oneDarkHighlighter];

export { oneDark };
