import { EditorState, Transaction, ChangeSet, Facet, Extension } from "@codemirror/next/state";
import { StyleModule } from "style-mod";
import { DecorationSet } from "./decoration";
import { EditorView } from "./editorview";
import { Attrs } from "./attributes";
import { Rect } from "./dom";
import { MakeSelectionStyle } from "./input";
export declare type Command = (target: EditorView) => boolean;
export declare const clickAddsSelectionRange: Facet<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
export declare const dragMovesSelection: Facet<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
export declare const mouseSelectionStyle: Facet<MakeSelectionStyle, readonly MakeSelectionStyle[]>;
export declare const exceptionSink: Facet<(exception: any) => void, readonly ((exception: any) => void)[]>;
export declare const updateListener: Facet<(update: ViewUpdate) => void, readonly ((update: ViewUpdate) => void)[]>;
export declare const inputHandler: Facet<(view: EditorView, from: number, to: number, text: string) => boolean, readonly ((view: EditorView, from: number, to: number, text: string) => boolean)[]>;
export declare function logException(state: EditorState, exception: any, context?: string): void;
export declare const editable: Facet<boolean, boolean>;
export interface PluginValue {
    update?(_update: ViewUpdate): void;
    destroy?(): void;
}
declare const isFieldProvider: unique symbol;
export declare class PluginFieldProvider<V> {
    private [isFieldProvider];
}
export declare class PluginField<T> {
    from<V extends PluginValue>(get: (value: V) => T): PluginFieldProvider<V>;
    static define<T>(): PluginField<T>;
    static scrollMargins: PluginField<Partial<Rect> | null>;
}
export declare const viewPlugin: Facet<ViewPlugin<any>, readonly ViewPlugin<any>[]>;
export interface PluginSpec<V extends PluginValue> {
    eventHandlers?: {
        [Type in keyof HTMLElementEventMap]?: (this: V, event: HTMLElementEventMap[Type], view: EditorView) => boolean | void;
    };
    decorations?: (value: V) => DecorationSet | readonly ((value: V) => DecorationSet)[];
    provide?: PluginFieldProvider<V> | readonly PluginFieldProvider<V>[];
}
export declare class ViewPlugin<V extends PluginValue> {
    extension: Extension;
    private constructor();
    static define<V extends PluginValue>(create: (view: EditorView) => V, spec?: PluginSpec<V>): ViewPlugin<V>;
    static fromClass<V extends PluginValue>(cls: {
        new (view: EditorView): V;
    }, spec?: PluginSpec<V>): ViewPlugin<V>;
}
export declare const pluginDecorations: PluginField<DecorationSet>;
export declare const domEventHandlers: PluginField<{
    plugin: PluginValue;
    handlers: {
        fullscreenchange?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        fullscreenerror?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        abort?: ((event: UIEvent, view: EditorView) => boolean | void) | undefined;
        animationcancel?: ((event: AnimationEvent, view: EditorView) => boolean | void) | undefined;
        animationend?: ((event: AnimationEvent, view: EditorView) => boolean | void) | undefined;
        animationiteration?: ((event: AnimationEvent, view: EditorView) => boolean | void) | undefined;
        animationstart?: ((event: AnimationEvent, view: EditorView) => boolean | void) | undefined;
        auxclick?: ((event: MouseEvent, view: EditorView) => boolean | void) | undefined;
        blur?: ((event: FocusEvent, view: EditorView) => boolean | void) | undefined;
        cancel?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        canplay?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        canplaythrough?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        change?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        click?: ((event: MouseEvent, view: EditorView) => boolean | void) | undefined;
        close?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        contextmenu?: ((event: MouseEvent, view: EditorView) => boolean | void) | undefined;
        cuechange?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        dblclick?: ((event: MouseEvent, view: EditorView) => boolean | void) | undefined;
        drag?: ((event: DragEvent, view: EditorView) => boolean | void) | undefined;
        dragend?: ((event: DragEvent, view: EditorView) => boolean | void) | undefined;
        dragenter?: ((event: DragEvent, view: EditorView) => boolean | void) | undefined;
        dragexit?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        dragleave?: ((event: DragEvent, view: EditorView) => boolean | void) | undefined;
        dragover?: ((event: DragEvent, view: EditorView) => boolean | void) | undefined;
        dragstart?: ((event: DragEvent, view: EditorView) => boolean | void) | undefined;
        drop?: ((event: DragEvent, view: EditorView) => boolean | void) | undefined;
        durationchange?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        emptied?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        ended?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        error?: ((event: ErrorEvent, view: EditorView) => boolean | void) | undefined;
        focus?: ((event: FocusEvent, view: EditorView) => boolean | void) | undefined;
        focusin?: ((event: FocusEvent, view: EditorView) => boolean | void) | undefined;
        focusout?: ((event: FocusEvent, view: EditorView) => boolean | void) | undefined;
        gotpointercapture?: ((event: PointerEvent, view: EditorView) => boolean | void) | undefined;
        input?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        invalid?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        keydown?: ((event: KeyboardEvent, view: EditorView) => boolean | void) | undefined;
        keypress?: ((event: KeyboardEvent, view: EditorView) => boolean | void) | undefined;
        keyup?: ((event: KeyboardEvent, view: EditorView) => boolean | void) | undefined;
        load?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        loadeddata?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        loadedmetadata?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        loadstart?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        lostpointercapture?: ((event: PointerEvent, view: EditorView) => boolean | void) | undefined;
        mousedown?: ((event: MouseEvent, view: EditorView) => boolean | void) | undefined;
        mouseenter?: ((event: MouseEvent, view: EditorView) => boolean | void) | undefined;
        mouseleave?: ((event: MouseEvent, view: EditorView) => boolean | void) | undefined;
        mousemove?: ((event: MouseEvent, view: EditorView) => boolean | void) | undefined;
        mouseout?: ((event: MouseEvent, view: EditorView) => boolean | void) | undefined;
        mouseover?: ((event: MouseEvent, view: EditorView) => boolean | void) | undefined;
        mouseup?: ((event: MouseEvent, view: EditorView) => boolean | void) | undefined;
        pause?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        play?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        playing?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        pointercancel?: ((event: PointerEvent, view: EditorView) => boolean | void) | undefined;
        pointerdown?: ((event: PointerEvent, view: EditorView) => boolean | void) | undefined;
        pointerenter?: ((event: PointerEvent, view: EditorView) => boolean | void) | undefined;
        pointerleave?: ((event: PointerEvent, view: EditorView) => boolean | void) | undefined;
        pointermove?: ((event: PointerEvent, view: EditorView) => boolean | void) | undefined;
        pointerout?: ((event: PointerEvent, view: EditorView) => boolean | void) | undefined;
        pointerover?: ((event: PointerEvent, view: EditorView) => boolean | void) | undefined;
        pointerup?: ((event: PointerEvent, view: EditorView) => boolean | void) | undefined;
        progress?: ((event: ProgressEvent<EventTarget>, view: EditorView) => boolean | void) | undefined;
        ratechange?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        reset?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        resize?: ((event: UIEvent, view: EditorView) => boolean | void) | undefined;
        scroll?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        securitypolicyviolation?: ((event: SecurityPolicyViolationEvent, view: EditorView) => boolean | void) | undefined;
        seeked?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        seeking?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        select?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        selectionchange?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        selectstart?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        stalled?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        submit?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        suspend?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        timeupdate?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        toggle?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        touchcancel?: ((event: TouchEvent, view: EditorView) => boolean | void) | undefined;
        touchend?: ((event: TouchEvent, view: EditorView) => boolean | void) | undefined;
        touchmove?: ((event: TouchEvent, view: EditorView) => boolean | void) | undefined;
        touchstart?: ((event: TouchEvent, view: EditorView) => boolean | void) | undefined;
        transitioncancel?: ((event: TransitionEvent, view: EditorView) => boolean | void) | undefined;
        transitionend?: ((event: TransitionEvent, view: EditorView) => boolean | void) | undefined;
        transitionrun?: ((event: TransitionEvent, view: EditorView) => boolean | void) | undefined;
        transitionstart?: ((event: TransitionEvent, view: EditorView) => boolean | void) | undefined;
        volumechange?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        waiting?: ((event: Event, view: EditorView) => boolean | void) | undefined;
        wheel?: ((event: WheelEvent, view: EditorView) => boolean | void) | undefined;
        copy?: ((event: ClipboardEvent, view: EditorView) => boolean | void) | undefined;
        cut?: ((event: ClipboardEvent, view: EditorView) => boolean | void) | undefined;
        paste?: ((event: ClipboardEvent, view: EditorView) => boolean | void) | undefined;
    };
}>;
export declare class PluginInstance {
    readonly value: PluginValue;
    readonly spec: ViewPlugin<any>;
    constructor(value: PluginValue, spec: ViewPlugin<any>);
    static create(spec: ViewPlugin<any>, view: EditorView): PluginInstance;
    takeField<T>(type: PluginField<T>, target: T[]): void;
    update(update: ViewUpdate): PluginInstance;
    destroy(view: EditorView): void;
    static dummy: PluginInstance;
}
export interface MeasureRequest<T> {
    read(view: EditorView): T;
    write(measure: T, view: EditorView): void;
    key?: any;
}
export declare const editorAttributes: Facet<Attrs, Attrs>;
export declare const contentAttributes: Facet<Attrs, Attrs>;
export declare const decorations: Facet<DecorationSet, readonly DecorationSet[]>;
export declare const styleModule: Facet<StyleModule, readonly StyleModule[]>;
export declare const enum UpdateFlag {
    Focus = 1,
    Height = 2,
    Viewport = 4,
    LineGaps = 8,
    Geometry = 16
}
export declare class ChangedRange {
    readonly fromA: number;
    readonly toA: number;
    readonly fromB: number;
    readonly toB: number;
    constructor(fromA: number, toA: number, fromB: number, toB: number);
    join(other: ChangedRange): ChangedRange;
    addToSet(set: ChangedRange[]): ChangedRange[];
    static extendWithRanges(diff: readonly ChangedRange[], ranges: number[]): readonly ChangedRange[];
}
export declare class ViewUpdate {
    readonly view: EditorView;
    readonly state: EditorState;
    readonly transactions: readonly Transaction[];
    readonly changes: ChangeSet;
    readonly prevState: EditorState;
    get viewportChanged(): boolean;
    get heightChanged(): boolean;
    get geometryChanged(): boolean;
    get focusChanged(): boolean;
    get docChanged(): boolean;
    get selectionSet(): boolean;
}
export {};
//# sourceMappingURL=extension.d.ts.map